/*
 *
 * Copyright 2018 Francesco Sorge <contact@francescosorge.com>.
 *
 * Licensed under the Apache License, Version 2.0 (the "License");
 * you may not use this file except in compliance with the License.
 * You may obtain a copy of the License at
 *
 *     http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 *
 */

#include <iostream>
#include <vector>

class Exam {
public:
    Exam(const std::string &pName, int pMark, int pCfu, bool pIsCompitino) {
        name = pName;
        mark = pMark;
        cfu = pCfu;
        isCompitino = pIsCompitino;
    }

    std::string getName() const {
        return name;
    }

    int getMark() const {
        return mark;
    }

    int getCfu() const {
        return cfu;
    }

    bool getIsCompitino() const {
        return isCompitino;
    }

private:
    std::string name;
    int mark, cfu;
    bool isCompitino;
};

class University {
public:
    explicit University(const std::string &pName) {
        name = pName;
    }

    std::string getName() const {
        return name;
    }

    void addExam(const Exam exam) {
        exams.emplace_back(exam);
    }

    void printExams() const {
        for (auto &exam : exams) {
            std::string type;
            if (exam.getIsCompitino()) {
                type = "Mid-exam";
            } else {
                type = "Exam";
            }

            std::cout << type << " of " << exam.getName() << " with mark " << exam.getMark() << " (" << exam.getCfu() << " CFU)" << std::endl;
        }
    }

    double calcAverage() const {
        if (exams.empty()) {
            return 0;
        }

        double sumMark = 0, sumCfu = 0;

        for (auto &exam: exams) {
            if (!exam.getIsCompitino()) {
                sumMark += exam.getMark() * exam.getCfu();
                sumCfu += exam.getCfu();
            }
        }

        return sumMark / sumCfu;
    }

private:
    std::string name;
    std::vector<Exam> exams;
};

class Student {
public:
    Student(const std::string &pName, const std::string &pSurname) {
        name = pName;
        surname = pSurname;
    }

    const std::string getName() const {
        return name;
    }

    const std::string getSurname() const {
        return surname;
    }

    void addUniversity(const std::string &pName) {
        university.emplace_back(University(pName));
    }

    void addExam(const std::string &pUniversity, const std::string &pExam, int mark, int cfu, bool isCompitino = false) {
        bool found = false;

        for(auto &current: university) {
            if (current.getName() == pUniversity) {
                found = true;
                current.addExam(Exam(pExam, mark, cfu, isCompitino));
                break;
            }
        }

        if (!found) {
            std::cout << "Cannot add this exam to the chosen university because it doesn't exist." << std::endl;
        }
    }

    double getAverage(const std::string &pUniversity) const {
        for(auto &current: university) {
            if (current.getName() == pUniversity) {
                return current.calcAverage();
            }
        }

        return 0.00;
    }

    void printUniversities() const {
        for(auto &current: university) {
            std::cout << current.getName() << std::endl;
        }
    }

    void printExams(const std::string &pUniversity) const {
        for(auto &current: university) {
            if (current.getName() == pUniversity) {
                current.printExams();
                break;
            }
        }
    }

    void printAverage(const std::string &pUniversity) const {
        double average = getAverage(pUniversity);

        if (average == 0) {
            std::cout << "You didn't do any exam at " << pUniversity << std::endl;
        } else {
            std::cout << "Your average at " << pUniversity << " is " << average << std::endl;
        }

    }

    void printAll() const {
        std::cout << "STATUS OF " << name << " " << surname << std::endl;

        for(auto &current: university) {
            std::cout << current.getName() << std::endl;
            printExams(current.getName());
            printAverage(current.getName());
            std::cout << "-------------------------------" << std::endl;
        }

        std::cout << std::endl;
    }

private:
    std::string name, surname;
    std::vector<University> university;
};

int main() {
    std::cout << "NEW STUDENT" << std::endl;
    std::string name, surname;
    std::cout << "Name: ";
    std::getline(std::cin, name);

    std::cout << "Surname: ";
    std::getline(std::cin, surname);

    std::cout << std::endl;

    Student me = Student(name, surname);

    //std::vector<Student> students;
    bool exit = false;

    std::cout << "MyUni - Developed by Francesco Sorge <contact@francescosorge.com>" << std::endl;
    //std::cout << "\t1. Aggiungi studente" << std::endl;
    std::cout << "\t2. Add university" << std::endl;
    std::cout << "\t3. Add exam" << std::endl;
    std::cout << "\t4. Print student's status" << std::endl;
    std::cout << "\t5. Exit" << std::endl;

    while (!exit) {
        std::cout << "Choose an option [2-5]: ";

        std::string option;
        std::getline(std::cin, option);

        /*if (option == "1") {
            std::string name, surname;
            std::getline(std::cin, name);
            std::getline(std::cin, surname);

            students.emplace_back(Student(name, surname));
        } else*/ if (option == "2") {
            std::cout << "Adding a new university. Type its name: ";
            std::string universityName;
            std::getline(std::cin, universityName);

            me.addUniversity(universityName);
        } else if (option == "3") {
            std::string universityName, examName;
            int mark, cfu;
            char compitino;
            bool isCompitino;

            std::cout << "Type the university's name where you did the exam: ";
            std::getline(std::cin, universityName);

            std::cout << "Exam name: ";
            std::getline(std::cin, examName);

            std::cout << "Mark: ";
            std::cin >> mark;

            std::cout << "CFU: ";
            std::cin >> cfu;

            std::cout << "Was it a mid-exam? (y/n) ";
            std::cin >> compitino;

            if (compitino == 'y') {
                isCompitino = true;
            } else {
                isCompitino = false;
            }

            me.addExam(universityName, examName, mark, cfu, isCompitino);
        } else if (option == "4") {
            me.printAll();
        } else if (option == "5") {
            std::cout << "Exiting MyUni..." << std::endl;
            exit = true;
        } else {
            std::cout << "Option not valid" << std::endl;
        }

        std::cout << std::endl;
    }

    return 0;
}